SystemOrganization addCategory: #'Git-Plumbing'!
SystemOrganization addCategory: #'Git-Tests'!

TestCase subclass: #FSUsage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!FSUsage methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 17:24'!
testReferencing
	| mem ref fileRef |
	mem := FSMemoryFilesystem new.
	ref := (mem working / 'dir').
	ref createDirectory.
	self assert: ref exists.
	
	fileRef := (ref / 'file').
	self deny: fileRef exists.
	fileRef writeStreamDo: [:stream | stream nextPutAll: 'foo'].
	self assert: fileRef exists.
	
	self assert: (mem working / 'dir') exists.
	self assert: (mem working / 'dir' / 'file') exists.
	self deny: (mem working / 'dir/file') exists! !

!FSUsage methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 17:37'!
testReferencingSubDirs
	| mem ref1 ref2 |
	mem := FSMemoryFilesystem new.
	ref1 := mem working / 'test' / 'dir'.
	ref2 := mem working / 'test' / 'dir'.
	self assert: ref1 = ref2.
	
	ref2 := mem working / 'test/dir'.
	self assert: ref1 species = ref2 species.
	self assert: ref1 filesystem = ref2 filesystem.
	self deny: ref1 path = ref2 path.
	self deny: ref1 = ref2! !

!FSUsage methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 18:05'!
testReferencingWithResolve
	| mem ref1 ref2 |
	mem := FSMemoryFilesystem new.
	ref1 := mem working / 'test'.
	ref1 := mem working resolve: 'test/blah'.
	ref2 := mem working / 'test' / 'blah'	.
	self assert: ref1 = ref2! !

TestCase subclass: #GitBasicTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitBasicTests methodsFor: 'tests' stamp: 'tonyg 2/23/2010 21:43'!
testStampReading
	| str stamp |
	str := 'Tony Garnock-Jones <tonygarnockjones@gmail.com> 1266614121 +1300'.
	stamp := GitStamp fromString: str.
	self assert: str = stamp printString.! !

TestCase subclass: #GitBlobTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitBlobTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:28'!
testRetrievingBytes
	| blob |
	blob := GitBlob bytes: 'testBlob'.
	self assert: blob bytes = ('testBlob' asByteArray).
	
	blob := GitBlob bytes: 3242.
	self assert: blob bytes = (3242 asByteArray).
	
	blob := GitBlob bytes: #[3 42 45 3].
	self assert: blob bytes = #[3 42 45 3]! !

TestCase subclass: #GitCommitTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/23/2010 19:13'!
testInitializeWithEverything
	| commit tree message author committer blob |
	blob := GitBlob bytes: 'aBlob'.
	message := 'a test message'.
	author := (GitStamp new
				email: 'mail@server.ext';
				name: 'Homer Simpson';
				timeStamp: TimeStamp now).
	committer := (GitStamp new
					email: 'homer@simpson.spr';
					name: 'Bart Simpson';
					timeStamp: TimeStamp now).
	tree := GitTree entries: {GitTreeEntry 
								fileName: 'filename'
								mode: GitTreeEntry fileMode
								objectSignature: blob signature}.
	
	commit := GitCommit 
				withTree: tree
				message: message
				author: author
				committer: committer.
	
	self assert: commit tree = tree.
	self assert: commit message = message.
	self assert: commit author = author.
	self assert: commit committer = committer! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:55'!
testPrintOn
	| message commit template |
	message := 'printOn test'.
	commit := GitCommit new.
	commit message: message.
	
	template := 'a GitCommit(''printOn test'')'.
	
	self assert: commit printString = template.! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:34'!
testRetrievingParents
	| commit collection parent1 parent2 parent3 |
	parent1 := GitCommit new.
	parent2 := GitCommit new.
	parent3 := GitCommit new.
	collection := OrderedCollection new.
	collection add: parent1; add: parent2; add: parent3.
	commit := GitCommit new.
	commit parents: collection.
	
	self assert: commit parents size = 3.
	self assert: commit parents first == parent1.
	self assert: commit parents second == parent2.
	self assert: commit parents third == parent3! !

TestCase subclass: #GitObjectLoaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitObjectLoaderTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/27/2010 20:28'!
testFor
	| repository loader |
	repository := GitRepository on: FSMemoryFilesystem new root.
	loader := GitObjectLoader for: repository.
	self assert: (loader instVarNamed: 'repository')= repository! !

!GitObjectLoaderTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/27/2010 20:25'!
testNew
	self should: [GitObjectLoader new] raise: Error! !

TestCase subclass: #GitObjectSignatureTests
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:54'!
authorEmail
	^'homer@nuke.com'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:55'!
authorName
	^'Homer Simpson'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 6/23/2010 21:34'!
authorTimeStamp
	^(TimeStamp fromUnixTime: 1274888648) offset: (Duration hours: 2)! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:54'!
referenceAuthor
	^GitStamp new
		name: self authorName;
		email: self authorEmail;
		timeStamp: self authorTimeStamp! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:45'!
referenceBlobHash
	^'e7cbb71a0d5b66bd7116563b395a98a81e50f988'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceBlobSize
	^8! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:36'!
referenceCommitHash
	^'0f02985452ad9beebb2af39649145fbf5d95aa31'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceContent
	^'testfile'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceFileName
	^'TESTFILE.txt'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:11'!
referenceMessage
	^'testfile commit'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 6/23/2010 19:59'!
referenceTreeEntryMode
	^8r100644! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:52'!
referenceTreeHash
	^'ad8473da921cc7878fe1106d3083b477d6592de4'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 11:43'!
referenceTreeSize
	^40! !

!GitObjectSignatureTests methodsFor: 'running' stamp: 'MaxLeske 5/14/2010 19:09'!
setUp
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: 'test.git').
	repository gitInitialize! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/23/2010 19:13'!
testByteSignatureCreation
	"reference commit created with GitX 0.7.1 on git 1.7.0"
	| fileRef stream testCommit tree blob |	
	fileRef := FSMemoryFilesystem new root resolve: 'TESTFILE.txt'.
	fileRef writeStreamDo: [:writer | writer nextPutAll: self referenceContent].
	stream := fileRef readStream.
	self assert:  stream contents size = self referenceBlobSize.	
	blob := GitBlob bytes: stream contents.
	
	tree := GitTree entries: {GitTreeEntry
								fileName: self referenceFileName
								mode: GitTreeEntry fileMode
								objectSignature: blob signature}.
	testCommit := GitCommit 
					withTree: tree
					message: self referenceMessage
					author: self referenceAuthor
					committer: self referenceAuthor.
	
	self assert: blob signature hexHash = (self referenceBlobHash).
	self assert: GitTreeEntry fileMode  = (self referenceTreeEntryMode).
	self assert: tree size = (self referenceTreeSize).
	self assert: tree signature hexHash = (self referenceTreeHash).
	"self assert: testCommit signature hexHash = (self referenceCommitHash)"! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 5/9/2010 11:42'!
testByteSignatureForBlob
	| stream bytes |
	stream := ReadStream on: (GitObjectSignature new byteSignatureFor: (GitBlob bytes: 'test')).
	
	bytes := stream upTo: Character space asInteger.
	self assert: bytes asString = 'blob'.
	
	bytes := stream upTo: 0.
	self assert: bytes asString asInteger = 4.
	
	bytes := stream upToEnd.
	self assert: bytes asString = 'test'! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/24/2010 12:33'!
testEquals
	| signature1 signature2 |
	signature1 := GitObjectSignature for: (GitBlob bytes: 'test1').
	signature2 := GitObjectSignature for: (GitBlob bytes: 'test2').
	self deny: signature1 = signature2! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 5/9/2010 11:44'!
testHexHashFromIntegerHash
	| hash |
	hash := SecureHashAlgorithm new hashMessage: 11.
	self assert: hash hex size = 39.
	hash := GitObjectSignature new hexHashFrom: hash.
	self assert: hash size = 40.
	self assert: hash asLowercase = hash.! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:48'!
testPrintOn
	| signature template |
	signature := GitObjectSignature from: 'bf8b4530d8d246dd74ac53a13471bba17941dff7'.
	template := 'a GitObjectSignature(bf8b4530d8d246dd74ac53a13471bba17941dff7)'.
	
	self assert: signature printString = template! !

TestCase subclass: #GitRepositoryTests
	instanceVariableNames: 'repository reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitRepositoryTests methodsFor: 'helpers' stamp: 'MaxLeske 4/20/2010 18:25'!
defaultStamp
	| str |
	str := 'Tony Garnock-Jones <tonygarnockjones@gmail.com> 1266614121 +1300'.
	^GitStamp fromString: str! !

!GitRepositoryTests methodsFor: 'running' stamp: 'MaxLeske 5/14/2010 19:12'!
setUp
	reference := (FSMemoryFilesystem new root resolve: 'test.git').
	repository := GitRepository on: reference.
	repository gitInitialize! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:49'!
testCommit
	| blob tree commit |
	blob := GitBlob bytes: 'hello'.
	repository storeObject: blob.
	
	tree := GitTree entries: { GitTreeEntry 
								fileName: 'testFile' 
								mode: GitTreeEntry fileMode
								objectSignature: blob signature }.
	repository storeObject: tree.
	
	commit := GitCommit new
		author: self defaultStamp;
		committer: self defaultStamp;
		tree: tree;
		message: 'initial commit';
		yourself.
	repository storeObject: commit.
	
	repository makeTag: (GitTag named: 'tagtest' for: commit message: 'First tag' tagger: self defaultStamp).
	repository makeTagNamed: 'lighttag' on: commit.
	repository updateRef: 'HEAD' to: commit signature! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 15:19'!
testCreateReferencedDirectory
	| filesystem |
	filesystem := FSMemoryFilesystem new.
	reference := filesystem root resolve: 'alpha/beta/gamma/repo.git'.
	repository := GitRepository on: reference.
	
	repository createReferencedDirectory.
	filesystem root allChildren do: [:each | self assert: each exists].
	
	filesystem := FSMemoryFilesystem new.
	reference := filesystem root resolve: '/alpha/beta/repo.git'.
	(filesystem root / 'alpha') createDirectory.
	repository := GitRepository on: reference.
	
	self 
		shouldnt: [repository createReferencedDirectory]
		raise: FSDirectoryExists
		description: 'The code should check if one of parent directories exists before trying to create it'.
	self assert: (filesystem root / 'alpha') exists.
	self assert: (filesystem root / 'alpha' / 'beta') exists.
	self assert: (filesystem root / 'alpha' / 'beta' / 'repo.git') exists.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 14:00'!
testDelete
	self assert: repository exists description: 'Need a repository to delete.'.
	repository delete.
	self deny: repository exists description: 'Deletion of repository failed.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/12/2010 14:27'!
testEquals
	| repo2 |
	repo2 := GitRepository on: FSMemoryFilesystem new working.
	self deny: repository = repo2.
	
	repo2 := GitRepository on: repository reference.
	self assert: repository = repo2! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:30'!
testExists
	repository := GitRepository on: (reference resolve: 'newRepo').
	self shouldnt: [ repository exists ].
	repository gitInitialize.
	self should: [ repository exists ]! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:31'!
testGitInitializeOnReferenceWithSubDirs
	reference := FSMemoryFilesystem new root resolve: 'alpha/beta/testRepo.git'.
	repository := GitRepository on: reference.
	self 
		shouldnt: [repository gitInitialize]
		raise: FSDirectoryDoesNotExist
		description: 'The reference of the repository points to a directory'
			, 'inside of a parent directory that does not exist.'
			, 'Create the parent directory first.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/24/2010 09:46'!
testInitializeOnDiskFilesystem
	reference := FSDiskFilesystem current working resolve: 'newRepo'.
	repository := GitRepository basicNew initializeOn: reference.
	self assert: (repository reference filesystem isKindOf: FSDiskFilesystem).
	self assert: (repository reference = reference)! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/24/2010 09:46'!
testInitializeOnMemoryFilesystem
	reference := FSMemoryFilesystem new working resolve: 'newRepo'.
	repository := GitRepository basicNew initializeOn: reference.
	self assert: (repository reference filesystem isKindOf: FSMemoryFilesystem).
	self assert: (repository reference = reference)! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:31'!
testIsInitialized
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: 'test.git').
	self deny: repository isInitialized.
	
	repository gitInitialize.
	self assert: repository isInitialized! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:45'!
testLooseObjectBytesAt
	| blob bytes |
	blob := repository storeObject: (GitBlob bytes: 'testBlob').
	bytes := repository looseObjectBytesAt: blob signature.

	self assert: bytes = (blob signature bytes)! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:47'!
testLooseObjectNames
	| blob names |
	blob := repository storeObject: (GitBlob bytes: 'test').
	names := repository looseObjectNames.
	self assert: names size = 1.
	self assert: names first species = (blob signature species).
	self assert: names first = (blob signature)! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 4/25/2010 19:57'!
testMakeTagNamedOn
	| tag objectName |
	objectName := repository storeObject: (GitBlob bytes: 'test').
	tag := repository makeTagNamed: 'testTag' on: objectName.
	
	self assert: (reference resolve: 'refs/tags/testTag') exists! !

!GitRepositoryTests methodsFor: 'class-tests' stamp: 'MaxLeske 4/28/2010 19:44'!
testNew
	self should: [GitRepository new] raise: Error! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:10'!
testObjectWithSignature
	| initialBlob loadedBlob blobSignature |
	initialBlob := GitBlob bytes: 'test'.
	repository storeObject: initialBlob.
	loadedBlob := repository objectWithSignature: initialBlob signature.
	self assert: initialBlob = loadedBlob! !

!GitRepositoryTests methodsFor: 'class-tests' stamp: 'MaxLeske 4/28/2010 19:44'!
testOnClass
	self should: [GitRepository on: FSDiskFilesystem]
		raise: Error
		description: 'Please provide an instance of a filesystem, not a class.'.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:10'!
testResolveRef
	| blob resolved |
	blob := repository storeObject: (GitBlob bytes: 'test').
	repository makeTagNamed: 'testTag' on: blob.
	resolved := repository resolveRef: 'testTag'.
	self assert: blob signature = resolved! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'Max Leske 4/25/2010 17:59'!
testSetupDirectoryStructure
	| dirs |
	dirs := #(
		'HEAD'
		'info'
		'objects'
		'objects/pack'
		'objects/info'
		'refs'
		'refs/heads'
		'refs/tags').
	dirs do: [ :dir | self assert: (repository reference resolveString: dir) exists ]! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:07'!
testStoreObjectBlob
	| blob storedObject signature hash |
	blob := GitBlob bytes: 'test'.
	signature := blob signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: blob.
	self assert: storedObject = blob! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:06'!
testStoreObjectCommit
	| tree commit signature hash storedObject |
	tree := GitTree new.
	commit := GitCommit new.
	commit
		author: self defaultStamp;
		committer: self defaultStamp;
		tree: tree;
		message: 'testCommit'.
	signature := commit signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: commit.
	self assert: storedObject = commit! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 5/26/2010 21:51'!
testStoreObjectCommitNoAuthor
	| commit |
	commit := GitCommit new.
	commit tree: GitTree new.
	self 
		should: [repository storeObject: commit] 
		raise: Error 
		whoseDescriptionIncludes: 'Author identity is required.'
		description: 'Author needs to be set before storing the commit.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 4/29/2010 18:43'!
testStoreObjectCommitNoTree
	| commit |
	commit := GitCommit new.
	self 
		should: [repository storeObject: commit] 
		raise: Error 
		whoseDescriptionIncludes: 'Cannot have a commit without a tree.'
		description: 'Every commit needs a tree. Make sure this is checked for.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:05'!
testStoreObjectTree
	| tree storedObject signature hash |
	tree := GitTree new.
	signature := tree signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: tree.
	self assert: storedObject = tree! !

TestCase subclass: #GitStorableObjectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitStorableObjectTests class methodsFor: 'package selection' stamp: 'MaxLeske 6/26/2010 22:12'!
packageNamesUnderTest
	^#('Git-Plumbing')! !

!GitStorableObjectTests methodsFor: 'helpers' stamp: 'MaxLeske 6/27/2010 20:04'!
buildMockFor: aGitObjectClass
	| repository tree fakeName |
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: 'test').
	repository gitInitialize.
	fakeName := GitObjectSignature for: (GitBlob bytes: 'test').
	tree := GitTree entries: { 
				GitTreeEntry 
					fileName: 'testFile' 
					mode: GitTreeEntry fileMode
					objectSignature: fakeName }.
	
	aGitObjectClass = GitBlob ifTrue:
		[^GitBlob bytes: 'test'].
	aGitObjectClass = GitCommit ifTrue:
		[^GitCommit new
			author: self defaultStamp;
			committer: self defaultStamp;
			tree: tree;
			message: 'initial commit';
			yourself.].
	aGitObjectClass = GitTag ifTrue:
		[^GitTag
			named: 'testTag'
			for: (self buildMockFor: GitCommit)
			message: 'aMessage'
			tagger: self defaultStamp].
	aGitObjectClass = GitTree ifTrue:
		[^GitTree
			entries: { 
				GitTreeEntry 
					fileName: 'testFile' 
					mode: GitTreeEntry fileMode
					objectSignature: fakeName }].
	self error: 'the class you provided needs to be instantieted to test #loadFrom'! !

!GitStorableObjectTests methodsFor: 'helpers' stamp: 'MaxLeske 4/28/2010 21:16'!
defaultStamp
	| str |
	str := 'Tony Garnock-Jones <tonygarnockjones@gmail.com> 1266614121 +1300'.
	^GitStamp fromString: str! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/24/2010 14:46'!
testForType
	GitStorableObject allSubclasses do: [:each | each typeCode isNil 
		ifFalse: [self assert: (GitStorableObject forType: each typeCode) = each]
		ifTrue: [self should: [GitStorableObject forType: each typeCode] raise: Error]]! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/27/2010 21:45'!
testLoadFrom
	| stream object loadedObject repository clazzes |
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: 'repo').
	repository gitInitialize.
	clazzes := GitStorableObject allSubclasses reject: [:each | each typeCode isNil].
	clazzes do:
		[:each | object := self buildMockFor: each.
		repository storeObject: object.
		stream := repository looseObjectStreamAt: object signature.
		loadedObject := GitStorableObject loadFrom: stream from: repository. 
		self assert: loadedObject isNil not.
		self assert: loadedObject species = (object species).
		self assert: loadedObject = object]! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/24/2010 14:46'!
testTypeCode
	self assert: GitStorableObject typeCode isNil! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/26/2010 22:11'!
testWithSignature
	| instance signature |
	signature := GitObjectSignature from: 234.
	instance := GitStorableObject withSignature: signature.
	self assert: instance signature = signature.
	self assert: instance signature == signature! !

TestCase subclass: #GitTaglikeObjectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitTaglikeObjectTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 20:48'!
testLoadingAssociations
	| taglikeObject |
	taglikeObject := GitTaglikeObject new.
	self assert: taglikeObject properties isEmpty.
	
	taglikeObject loadKey: 'keyname' value: 'keyobject'.
	self assert: taglikeObject properties first species = Association.
	self assert: taglikeObject properties first key = 'keyname'
	! !

!GitTaglikeObjectTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 20:44'!
testRetrievingProperties
	| object collection |
	self assert: GitTaglikeObject new properties isEmpty.
	
	collection := OrderedCollection new.
	collection add: Object new.
	object := GitTaglikeObject new.
	object properties: collection.
	
	self assert: object properties = collection.
	self assert: object properties isEmpty not! !

TestCase subclass: #GitTreeEntryTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitTreeEntryTests methodsFor: 'tests' stamp: 'MaxLeske 6/24/2010 09:34'!
testSize
	| treeEntry fileName mode signature sizeToBe |
	fileName := 'testFile'.
	mode := GitTreeEntry printMode: GitTreeEntry fileMode.
	signature := (GitBlob bytes: 'fooBlob') signature.
	treeEntry := GitTreeEntry 
					fileName: fileName 
					mode: GitTreeEntry fileMode
					objectSignature: signature.
					
	sizeToBe := 
		(mode size) +
		(Character space asInteger asByteArray size) +
		(fileName size) +
		(0 decimalDigitLength) +
		(signature size).
	
	self assert: signature size = 20.
	self assert: treeEntry size = sizeToBe.! !

TestCase subclass: #GitTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Tests'!

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 6/26/2010 22:48'!
testAt
	| tree entry blob repo |
	blob := GitBlob bytes: 'testblob'.
	repo := GitRepository on: (FSMemoryFilesystem new root resolve: 'testRepo').
	repo storeObject: blob.
	entry := GitTreeEntry
		fileName: 'testFile'
		mode: GitTreeEntry fileMode
		objectSignature: blob signature.
	tree := GitTree in: repo.
	tree entries: {entry}.
	
	self assert: (tree at: entry fileName) = blob! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 6/26/2010 22:30'!
testNoEntries
	self assert: (GitTree new entries) species = Dictionary.
	self assert: (GitTree new entries) isEmpty! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:22'!
testRemoveEntry
	| tree collection entry1 entry2 entry3 |
	entry1 := GitTreeEntry
		fileName: 'entry 1'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 1).
	entry2 := GitTreeEntry
		fileName: 'entry 2'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 2).
	entry3 := GitTreeEntry
		fileName: 'entry 3'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 3).
	collection := OrderedCollection new add: entry1; add: entry2; add: entry3; yourself.
	tree := GitTree entries: collection.
	
	self assert: tree entries size = 3.
	self assert: (tree entries includesKey: entry1 fileName).
	self assert: (tree entries includesKey: entry2 fileName).
	self assert: (tree entries includesKey: entry3 fileName).
	
	tree removeEntryAt: entry2 fileName.
	self assert: tree entries size = 2.
	self deny: (tree entries includesKey: entry2 fileName).
	
	tree removeEntryAt: entry3 fileName.
	self assert: tree entries size = 1.
	self deny: (tree entries includesKey: entry3 fileName).
	
	tree removeEntryAt: entry1 fileName.
	self assert: tree entries isEmpty! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 6/26/2010 22:29'!
testSettingEntries
	| collection tree entry1 entry2 |
	collection := OrderedCollection new.
	entry1 := GitTreeEntry 
		fileName: 'object1' 
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 234).
	entry2 := GitTreeEntry 
		fileName: 'object2' 
		mode: GitTreeEntry dirMode
		objectSignature: (GitObjectSignature from: 564).
	collection add: entry1; add: entry2.
	tree := GitTree entries: collection.
	
	self assert: (tree entries at: entry1 fileName)= entry1.
	self assert: (tree entries at: entry2 fileName) = entry2.
	self deny: entry1 = entry2! !

Object subclass: #GitDeltaInterpreter
	instanceVariableNames: 'source instructions output commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitDeltaInterpreter methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 12:01'!
instructions
	^ instructions! !

!GitDeltaInterpreter methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 12:01'!
instructions: anObject
	instructions := anObject! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:09'!
readVariableInteger
	| val shift b |
	val := 0.
	shift := 0.
	[
		b := commands next.
		val := val bitOr: ((b bitAnd: 127) bitShift: shift).
		shift := shift + 7.
	] doWhileFalse: [(b bitAnd: 128) isZero].
	^ val! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:20'!
run
	[commands atEnd] whileFalse: [self runNextCommand]! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:25'!
runAgainst: aByteArray
	source := aByteArray.
	
	commands := ReadStream on: instructions.
	source size = self readVariableInteger ifFalse: [self error: 'Delta source size mismatch'].
	output := WriteStream on: (ByteArray new: self readVariableInteger).
	self run.

	^ output contents! !

!GitDeltaInterpreter methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 12:19'!
runNextCommand
	| cmd |
	cmd := commands next.
	cmd = 0 ifTrue: [self error: 'unexpected delta opcode 0'].
	cmd < 128
		ifTrue: [output nextPutAll: (commands next: cmd)]
		ifFalse: [
			| off size |

			off := 0.
			(cmd bitAnd: 2r00000001) isZero ifFalse: [off := commands next].
			(cmd bitAnd: 2r00000010) isZero ifFalse: [off := off bitOr: (commands next bitShift: 8)].
			(cmd bitAnd: 2r00000100) isZero ifFalse: [off := off bitOr: (commands next bitShift: 16)].
			(cmd bitAnd: 2r00001000) isZero ifFalse: [off := off bitOr: (commands next bitShift: 24)].

			size := 0.
			(cmd bitAnd: 2r00010000) isZero ifFalse: [size := commands next].
			(cmd bitAnd: 2r00100000) isZero ifFalse: [size := size bitOr: (commands next bitShift: 8)].
			(cmd bitAnd: 2r01000000) isZero ifFalse: [size := size bitOr: (commands next bitShift: 16)].
			size = 0 ifTrue: [size := 16r10000].
			
			output nextPutAll: (source copyFrom: off + 1 to: off + size)]! !

Object subclass: #GitObject
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:47'!
in: aGitRepository
	^self new repository: aGitRepository; yourself! !

!GitObject methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:14'!
loadFrom: aStream
	self subclassResponsibility! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 16:19'!
repository: aGitRipository
	repository := aGitRipository! !

!GitObject methodsFor: 'saving' stamp: 'MaxLeske 6/24/2010 14:50'!
saveTo: aStream
	self subclassResponsibility! !

!GitObject methodsFor: 'testing' stamp: 'MaxLeske 6/24/2010 14:47'!
size
	| stream |
	stream := GitStream new.
	self saveTo: stream.
	^stream size! !

GitObject subclass: #GitObjectSignature
	instanceVariableNames: 'signature integerHash'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 4/29/2010 18:05'!
for: aGitObject
	^self new computeSignatureFor: aGitObject! !

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 5/9/2010 13:13'!
from: aHexHash
	^self new from: aHexHash! !

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/24/2010 11:04'!
loadFrom: aStream
	| hash |
	hash := (aStream upTo: Character lf asInteger).
	^self from: hash hex! !

!GitObjectSignature methodsFor: 'comparing' stamp: 'MaxLeske 5/9/2010 15:15'!
= other
	^self integerHash = other integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 6/23/2010 19:58'!
byteSignatureFor: aGitObject
	| stream |
	stream := GitStream new.
	"format for object hash (see manual -> 'Object storage format'):
	<ascii type without space> + <space> + <ascii decimal size> + <byte\0> + <binary object data>"
	stream
		nextPutAll: aGitObject typeCode;
		space;
		nextPutAll: aGitObject byteData size asString; "ASCII value for $0 is not 0 but 48!!"
		nullByte;
		nextPutAll: aGitObject byteData.
	^stream contents! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 4/29/2010 19:49'!
bytes
	^signature! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/8/2010 20:41'!
computeSignatureFor: aGitObject
	signature := self byteSignatureFor: aGitObject.
	integerHash := self integerHashFrom: signature! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/9/2010 13:32'!
from: aHexHash
	integerHash := Integer readFrom: aHexHash asString asUppercase base: 16! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 11:09'!
hexHash
	^self hexHashFrom: integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/9/2010 14:38'!
hexHashFrom: aByteString
	"#hex will omit a leading 0. If that's the case insert leading 0."
	^aByteString asByteArray hex asLowercase forceTo: 40 paddingStartWith: $0! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 5/8/2010 20:42'!
integerHash
	^integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 4/29/2010 18:17'!
integerHashFrom: byteArray
	^SecureHashAlgorithm new hashMessage: byteArray! !

!GitObjectSignature methodsFor: 'printing' stamp: 'MaxLeske 5/8/2010 20:42'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self hexHash; nextPut: $).! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 10:58'!
saveTo: aStream
	aStream nextPutBytes: self integerHash asByteArray! !

GitObject subclass: #GitStorableObject
	instanceVariableNames: 'signature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

GitStorableObject subclass: #GitBlob
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitBlob class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 16:02'!
bytes: aByteArray
	^ self new initializeBytes: aByteArray asByteArray! !

!GitBlob class methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 07:19'!
typeCode
	^ 'blob'! !

!GitBlob methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 07:59'!
bytes
	^ bytes! !

!GitBlob methodsFor: 'initialize-release' stamp: 'tonyg 2/23/2010 13:31'!
initializeBytes: aByteArray
	bytes := aByteArray! !

!GitBlob methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:15'!
loadFrom: aStream
	bytes := aStream contents.
	^self! !

!GitBlob methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 15:36'!
saveTo: aStream
	aStream nextPutAll: bytes.! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 5/26/2010 22:14'!
forType: aString  
 ^ self allSubclasses
       detect: [ :class | aString isNil not and: [class typeCode = aString] ]
       ifNone: [ self error: 'Invalid type code: ' , aString printString ]! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:45'!
loadFrom: aStream from: aGitRepository
	| code length limitedStream object |
	code := (aStream upTo: Character space asInteger) asString.
	length := (aStream upTo: 0) asString asInteger.
	limitedStream := ReadStream on: (aStream next: length).
	object := (self forType: code) in: aGitRepository.
	^object loadFrom: limitedStream! !

!GitStorableObject class methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 12:24'!
typeCode
	"Override in subclasses, returning non-nil where you want them to be constructible from a git object store."
	^ nil! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 17:07'!
withSignature: aGitSignature
	^self new withSignature: aGitSignature! !

!GitStorableObject methodsFor: 'comparing' stamp: 'MaxLeske 4/28/2010 21:25'!
= other
	^self signature = other signature! !

!GitStorableObject methodsFor: 'accessing' stamp: 'Max Leske 4/25/2010 13:05'!
byteData
	| stream |
	stream := GitStream new.
	self saveTo: stream.
	^stream contents! !

!GitStorableObject methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 19:15'!
signature
	^signature ifNil: [signature := GitObjectSignature for: self]! !

!GitStorableObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 15:47'!
typeCode
	^ self class typeCode! !

!GitStorableObject methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 17:07'!
withSignature: aGitObjectSignature
	signature := aGitObjectSignature! !

GitStorableObject subclass: #GitTaglikeObject
	instanceVariableNames: 'message properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

GitTaglikeObject subclass: #GitCommit
	instanceVariableNames: 'tree parents author committer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitCommit class methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 22:21'!
typeCode
	^ 'commit'! !

!GitCommit class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 22:06'!
withDefaultStamps
	| s |
	s := GitStamp now.
	^ self new author: s; committer: s! !

!GitCommit class methodsFor: 'instance creation' stamp: 'MaxLeske 5/25/2010 19:59'!
withTree: aGitTree message: aString author: aGitStampForAuthor committer: aGitStampForCommitter
	^self new
		tree: aGitTree;
		message: aString;
		author: aGitStampForAuthor;
		committer: aGitStampForCommitter;
		yourself
		! !

!GitCommit methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:54'!
author
	^ author! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:52'!
author: aGitStamp
	author := aGitStamp! !

!GitCommit methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:54'!
committer
	^ committer! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:51'!
committer: aGitStamp
	committer := aGitStamp! !

!GitCommit methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/26/2010 17:14'!
loadKey: k value: v
	k = 'tree' ifTrue: [tree := repository objectNamed: v. ^ self].
	k = 'parent' ifTrue: [
		parents ifNil: [parents := OrderedCollection new].
		parents add: (repository objectNamed: v).
		^ self].
	k = 'author' ifTrue: [author := GitStamp fromString: v. ^ self].
	k = 'committer' ifTrue: [committer := GitStamp fromString: v. ^ self].
	"Otherwise it's treated generically."
	^ super loadKey: k value: v! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 17:15'!
parents
	^ parents ifNil: [OrderedCollection new]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 20:04'!
parents: aGitCommitCollection
	^ parents := aGitCommitCollection! !

!GitCommit methodsFor: 'printing' stamp: 'MaxLeske 6/27/2010 19:52'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: '(''';
		nextPutAll: self message;
		nextPutAll: ''')' ! !

!GitCommit methodsFor: 'as yet unclassified' stamp: 'MaxLeske 5/26/2010 20:08'!
saveFixedKeysTo: aStream
	"Looking at builtin-commit-tree.c, the order of properties is fixed:
	tree, parents, author, committer, everything else, message."
	self saveKey: 'tree' value: tree signature hexHash to: aStream.
	self parents do: [:each | self saveKey: 'parent' value: each signature hexHash to: aStream].
	self saveKey: 'author' value: author printString to: aStream.
	self saveKey: 'committer' value: committer printString to: aStream.! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 15:48'!
tree
	^tree! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:49'!
tree: aGitTree
	tree := aGitTree! !

!GitCommit methodsFor: 'as yet unclassified' stamp: 'MaxLeske 5/26/2010 21:47'!
validate
	tree ifNil: [self error: 'Cannot have a commit without a tree.'].
	tree species = GitTree ifFalse: [self error: 'Commit needs to hold an instance of a GitTree.'].
	author ifNil: [self error: 'Author identity is required.'].
	committer ifNil: [self error: 'Committer identity is required.'].
	super validate.! !

GitTaglikeObject subclass: #GitTag
	instanceVariableNames: 'tagName tagger object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitTag class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/27/2010 19:59'!
named: tagName for: aGitStorableObject message: message
	^ self named: tagName for: aGitStorableObject message: message tagger: GitStamp now! !

!GitTag class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/27/2010 20:00'!
named: tagName for: aGitStorableObject message: message tagger: aGitStamp
	^ self new
		tagName: tagName;
		object: aGitStorableObject;
		message: message;
		tagger: aGitStamp;
		yourself! !

!GitTag class methodsFor: 'as yet unclassified' stamp: 'tonyg 2/25/2010 16:16'!
typeCode
	^ 'tag'! !

!GitTag methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/27/2010 20:11'!
loadKey: k value: v
	k = 'object' ifTrue: [object := repository objectNamed: v. ^ self].
	k = 'type' ifTrue: [self assert: object typeCode = v. ^ self].
	k = 'tag' ifTrue: [tagName := v. ^ self].
	k = 'tagger' ifTrue: [tagger := GitStamp fromString: v. ^ self].
	"Otherwise it's treated generically."
	^ super loadKey: k value: v! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 20:01'!
object: aGitStorableObject
	object := aGitStorableObject! !

!GitTag methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/27/2010 20:03'!
saveFixedKeysTo: aStream
	"From builtin-mktag.c:
	A signature file has a very simple fixed format: four lines
	of ""object <sha1>"" + ""type <typename>"" + ""tag <tagname>"" +
	""tagger <committer>"", followed by a blank line, a free-form tag
	message and a signature block that git itself doesn't care about,
	but that can be verified with gpg or similar."
	self saveKey: 'object' value: object signature hexHash to: aStream.
	self saveKey: 'type' value: object typeCode to: aStream.
	self saveKey: 'tag' value: tagName to: aStream.
	self saveKey: 'tagger' value: tagger printString to: aStream.! !

!GitTag methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:30'!
tagName
	^ tagName! !

!GitTag methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:30'!
tagName: anObject
	tagName := anObject! !

!GitTag methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:30'!
tagger
	^ tagger! !

!GitTag methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:30'!
tagger: anObject
	tagger := anObject! !

!GitTag methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/27/2010 20:02'!
validate
	object ifNil: [self error: 'Cannot have a tag without a tagged object.'].
	tagName ifNil: [self error: 'Tag name is required.'].
	tagger ifNil: [self error: 'Tagger identity is required.'].
	super validate.! !

!GitTaglikeObject methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 21:53'!
loadFrom: aStream
	| key value |
	[aStream peek = Character lf asInteger] whileFalse: [
		key := (aStream upTo: Character space asInteger) asString.
		value := (aStream upTo: Character lf asInteger) asString.
		self loadKey: key value: value].
	aStream next.
	message := aStream upToEnd asString.
	^self
! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/26/2010 16:15'!
loadKey: k value: v
	self properties add: k -> v.! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/25/2010 16:22'!
message
	^ message! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/25/2010 16:22'!
message: aString
	message := aString! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/25/2010 16:22'!
properties
	^ properties ifNil: [properties := OrderedCollection new]! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:23'!
properties: anObject
	^ properties := anObject! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/25/2010 16:27'!
saveFixedKeysTo: aStream
	self subclassResponsibility! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'MaxLeske 4/23/2010 13:50'!
saveKey: key value: value to: aStream
	aStream 
		nextPutAll: key; space; 
		nextPutAll: value; lf.! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/26/2010 16:38'!
saveTo: aStream
	self validate.
	self saveFixedKeysTo: aStream.
	properties ifNotNil: [properties do: [:assoc |
			self saveKey: assoc key value: assoc value to: aStream]].
	aStream lf.
	aStream nextPutAll: message.! !

!GitTaglikeObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/25/2010 16:34'!
validate
	"Override in subclasses to provide pre-save validation checks.
	Don't forget to call super validate, though!!"
	message ifNil: [self error: 'Message is required.'].! !

GitStorableObject subclass: #GitTree
	instanceVariableNames: 'entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitTree class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 6/26/2010 22:42'!
entries: aCollection
	^self new entries: aCollection! !

!GitTree class methodsFor: 'as yet unclassified' stamp: 'tonyg 2/22/2010 07:17'!
typeCode
	^ 'tree'! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:38'!
at: aName
	^ repository objectWithSignature: (self entryAt: aName) objectSignature! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:28'!
entries
	^ entries ifNil: [Dictionary new]! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:42'!
entries: aCollection
	aCollection do: [:each | self putEntry: each].! !

!GitTree methodsFor: 'private' stamp: 'tonyg 2/23/2010 19:10'!
entryAt: aName
	^ entries at: aName! !

!GitTree methodsFor: 'initialize-release' stamp: 'tonyg 2/23/2010 20:18'!
initialize
	super initialize.
	entries := Dictionary new.! !

!GitTree methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:35'!
loadFrom: aStream
	| entry |
	[aStream atEnd] 
		whileFalse: [entry := (GitTreeEntry in: repository) loadFrom: aStream.
			self putEntry: entry].
	^self! !

!GitTree methodsFor: 'private' stamp: 'tonyg 2/23/2010 19:11'!
putEntry: anEntry
	entries at: anEntry fileName put: anEntry! !

!GitTree methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 19:14'!
removeEntryAt: aFileName
	entries removeKey: aFileName! !

!GitTree methodsFor: 'saving' stamp: 'tonyg 2/23/2010 15:41'!
saveTo: aStream
	entries valuesDo: [:e | e saveTo: aStream].! !

!GitTree methodsFor: 'testing' stamp: 'MaxLeske 6/8/2010 22:36'!
size
	| total |
	total := 0.
	self entries do: [:each | total := total + (each size)].
	^total! !

GitObject subclass: #GitTreeEntry
	instanceVariableNames: 'fileName mode objectSignature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitTreeEntry class methodsFor: 'private' stamp: 'MaxLeske 6/23/2010 19:10'!
dirMode
	^8r040000! !

!GitTreeEntry class methodsFor: 'private' stamp: 'MaxLeske 6/23/2010 19:13'!
fileMode
	^8r100644! !

!GitTreeEntry class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 5/16/2010 16:09'!
fileName: aString mode: anInteger objectSignature: aGitObjectSignature
	^ self new initializeFileName: aString mode: anInteger signature: aGitObjectSignature! !

!GitTreeEntry class methodsFor: 'printing' stamp: 'MaxLeske 6/23/2010 19:16'!
printMode: octalMode
	^octalMode printStringBase: 8 length: 6  padded: 0! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 08:08'!
fileName
	^ fileName! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'MaxLeske 5/9/2010 15:02'!
initializeFileName: aString mode: anInteger signature: aGitObjectSignature
	fileName := aString.
	mode := anInteger.
	objectSignature := aGitObjectSignature! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:18'!
loadFrom: aStream
	mode := Integer readFrom: (ReadStream on: (aStream upTo: Character space asInteger) asString) base: 8.
	fileName := (aStream upTo: 0) asString.
	objectSignature := GitObjectSignature loadFrom: aStream.
	^self! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 08:08'!
mode
	^ mode! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 13:03'!
objectSignature
	^ objectSignature! !

!GitTreeEntry methodsFor: 'printing' stamp: 'MaxLeske 6/23/2010 19:17'!
printMode
	^self class printMode: self mode! !

!GitTreeEntry methodsFor: 'saving' stamp: 'MaxLeske 6/23/2010 19:18'!
saveTo: aStream.
	aStream 
		nextPutAll: self printMode;
		space;
		nextPutAll: fileName;
		nullByte.
	objectSignature saveTo: aStream.! !

Object subclass: #GitObjectLoader
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 16:03'!
for: aGitRepository
	^self basicNew initializeOn: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 16:04'!
new
	self error: 'please use the constructor methods.'! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 12:11'!
objectFrom: aStream from: aGitRepository
	^self basicNew objectFrom: aStream from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 11:28'!
objectNamed: aGitObjectSignature from: aGitRepository
	^self basicNew objectNamed: aGitObjectSignature from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 13:16'!
objectWithSignature: aGitObjectSignature from: aGitRepository
	^self basicNew objectWithSignature: aGitObjectSignature from: aGitRepository! !

!GitObjectLoader methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:04'!
initializeOn: aGitRepository
	repository := aGitRepository! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 17:03'!
objectFrom: aStream
	repository ifNil: [self error: 'Instance variable #repository needs to be initialized.'].
	^GitStorableObject loadFrom: aStream from: self! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 16:57'!
objectFrom: aStream from: aGitRepository
	repository := aGitRepository.
	^self objectFrom: aStream! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:43'!
objectNamed: aHexHash
	repository ifNil: [self error: 'instance variable #repository needs to be initialized.'].
	^self objectWithSignature: (GitObjectSignature from: aHexHash)! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:47'!
objectNamed: aHexHash from: aGitRepository
	repository := aGitRepository.
	^self objectNamed: aHexHash! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:38'!
objectWithSignature: aGitObjectSignature
	repository ifNil: [self error: 'Instance variable #repository needs to be initialized.'].
	
	^ [ self objectFrom: (repository looseObjectStreamAt: aGitObjectSignature) ] 
		on: FSFileDoesNotExist
		do: [ :ex | ex return: (repository packedObjectAtSignature: aGitObjectSignature) ]! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:44'!
objectWithSignature: aGitObjectSignature from: aGitRepository
	repository := aGitRepository.
	^self objectWithSignature: aGitObjectSignature! !

Object subclass: #GitPack
	instanceVariableNames: 'idx basePath fanoutTable objectCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 15:42'!
basePath
	^ basePath! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:31'!
basePath: aString
	basePath := aString.
	idx := FileStream readOnlyFileNamed: basePath, '.idx'.
	idx binary.
	((idx next: 8) = #(255 116 79 99 0 0 0 2) asByteArray)
		ifFalse: [self error: 'Bad pack index header'].
	fanoutTable := idx next: 1024. "256 4-byte big-endian entries."
	objectCount := self fanoutTableAt: 255.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:43'!
crcAt: index
	"1024 fanout, 8 header, minus 4 for smalltalk's 1-based indexing"
	idx position: 1028 + (objectCount * 20) + (index * 4).
	^ (idx next: 4) asInteger.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:31'!
fanoutTableAt: fanoutIndex
	^ fanoutTable unsignedLongAt: (fanoutIndex * 4) + 1 bigEndian: true.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/24/2010 00:32'!
objectAt: sha1String
	| p |
	p := self packedObjectAt: sha1String.
	p ifNotNil: [^ p object].
	^ nil! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 10:50'!
offsetAt: index
	| o |
	"1024 fanout, 8 header. We're still using zero-based indexing here."
	idx position: 1032 + (objectCount * 24) + (index * 4).
	o := (idx next: 4) asInteger.
	(o >= 16r80000000)
		ifFalse: [^ o]
		ifTrue: [
			"the table offset is zero-based here because it comes from git, not smalltalk"
			idx position: 1032 + (objectCount * 28) + ((o bitAnd: 16r7FFFFFFF) * 8).
			^ (idx next: 8) asInteger]! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 19:56'!
packedObjectAt: sha1String
	| i o |
	i := self tableIndexForSha: sha1String.
	i ifNil: [^ nil].
	o := self offsetAt: i.
	^ self packedObjectAtOffset: o! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 19:56'!
packedObjectAtOffset: offset 
	| p c |
	p := FileStream readOnlyFileNamed: basePath , '.pack'.
	p binary.
	p position: offset.
	c := GitPackedObject new pack: self.
	c loadFrom: p.
	^ c! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 10:49'!
tableIndexForSha: s
	| fanoutIndex sNum index low high test |
	sNum := Integer readFrom: (ReadStream on: s asUppercase) base: 16.
	fanoutIndex := sNum bitShift: -152. "160 bit hash. We want the upper 8 bits."

	"We are searching using a zero-based index, since position: on the file is zero-based."
	low := (fanoutIndex > 0) ifTrue: [self fanoutTableAt: fanoutIndex - 1] ifFalse: [0].
	high := (self fanoutTableAt: fanoutIndex) - 1. "binary-search wants an inclusive range"

	[index := high + low // 2.
	low > high] whileFalse:[
		idx position: index * 20 + 1032. "256 * 4 for the fanoutTable, plus 8 for the header"
		test := (idx next: 20) asInteger.
		test = sNum 
			ifTrue:[^index]
			ifFalse:[sNum > test
				ifTrue: [low := index + 1]
				ifFalse: [high := index - 1]]].
	^ nil! !

Object subclass: #GitPackedObject
	instanceVariableNames: 'pack file decodeClass length headerPos unpackedBytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
applyDelta: delta to: base
	decodeClass := base decodeClass.
	unpackedBytes := delta runAgainst: base unpackedBytes.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
decodeClass
	^ decodeClass! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:04'!
loadFrom: aStream
	| b type shift |
	file := aStream.
	headerPos := file position.
	b := file next.
	type := (b bitShift: -4) bitAnd: 7.
	length := b bitAnd: 15.
	shift := 4.
	[(b bitAnd: 128) isZero] whileFalse: [
		b := file next.
		length := length bitOr: ((b bitAnd: 127) bitShift: shift).
		shift := shift + 7].
	self unpackContentType: type.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:10'!
object
	^ decodeClass new loadFrom: (ReadStream on: unpackedBytes)! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 11:05'!
pack
	^ pack! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 11:05'!
pack: anObject
	pack := anObject! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:07'!
readDelta
	^ GitDeltaInterpreter new instructions: (ZLibReadStream on: file) contents.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:04'!
unpackContentType: type
	type = 1 "OBJ_COMMIT" ifTrue: [decodeClass := GitCommit. ^ self unpackSimpleContent].
	type = 2 "OBJ_TREE" ifTrue: [decodeClass := GitTree. ^ self unpackSimpleContent].
	type = 3 "OBJ_BLOB" ifTrue: [decodeClass := GitBlob. ^ self unpackSimpleContent].
	type = 4 "OBJ_TAG" ifTrue: [self error: 'Packed tags aren''t supported yet.'].
	type = 6 "OBJ_OFS_DELTA" ifTrue: [^ self unpackOfsDeltaContent].
	type = 7 "OBJ_REF_DELTA" ifTrue: [^ self unpackRefDeltaContent].
	self error: 'Invalid pack object type ', type printString.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:07'!
unpackOfsDeltaContent
	| offset b base delta |
	b := file next.
	offset := b bitAnd: 127.
	[(b bitAnd: 128) isZero] whileFalse: [
		b := file next.
		"This is bizarre, but it's what pack-format.txt and the Git source says is happening."
		offset := ((offset + 1) bitShift: 7) bitOr: (b bitAnd: 127)].

	delta := self readDelta.
	base := pack packedObjectAtOffset: headerPos - offset.
	self applyDelta: delta to: base.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 22:12'!
unpackRefDeltaContent
	| s delta base |
	s := (GitObjectSignature loadFrom: file) resolve.
	delta := self readDelta.
	base := pack packedObjectAt: s.
	self applyDelta: delta to: base.! !

!GitPackedObject methodsFor: 'as yet unclassified' stamp: 'tonyg 2/23/2010 20:05'!
unpackSimpleContent 
	unpackedBytes := (ZLibReadStream on: file) contents.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
unpackedBytes
	^ unpackedBytes! !

Object subclass: #GitRepository
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitRepository commentStamp: 'MaxLeske 4/2/2010 10:59' prior: 0!
A GitRepository is xxxxxxxxx.

Instance Variables
	reference:		<Object>

reference
	I'm an instance of FSReference. I know where the repository is located and on what file system
!

!GitRepository class methodsFor: 'instance creation' stamp: 'MaxLeske 4/13/2010 15:11'!
new
	self error: 'This class should be istantiated with a memory type.'! !

!GitRepository class methodsFor: 'initialize-release' stamp: 'MaxLeske 4/23/2010 17:38'!
on: anFSReference
	| isClass |
	isClass := (anFSReference isKindOf: Class).
	isClass ifTrue: [self error: 'Please provide an instance of a filesystem, not a class.'].
	^self basicNew initializeOn: anFSReference
	! !

!GitRepository methodsFor: 'comparing' stamp: 'MaxLeske 5/12/2010 14:26'!
= other
	^self reference = (other reference)! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 5/13/2010 15:15'!
createReferencedDirectory
	| refWalker dirList |
	refWalker := reference.
	dirList := OrderedCollection new.
	
	[refWalker ~= (reference filesystem root)]
		whileTrue: [dirList addFirst: refWalker. refWalker := refWalker parent].
	
	dirList do: [:each | each exists ifFalse: [each createDirectory]]
	! !

!GitRepository methodsFor: 'operations' stamp: 'MaxLeske 4/2/2010 12:01'!
delete
	self reference delete! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 4/2/2010 12:02'!
exists
	^self reference exists! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 5/12/2010 13:45'!
gitDirectories
	^#('objects' 
		'objects/pack' 
		'objects/info' 
		'refs' 'refs/heads' 
		'refs/tags' 
		'info')! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 22:02'!
gitInitialize
	self exists
		ifTrue: [ ^ self ].
	self createReferencedDirectory.
	self gitDirectories do: [ :dir | (reference resolve: dir) createDirectory ].
	reference / 'HEAD'
		writeStreamDo: [ :stream | 
			stream
				nextPutAll: 'ref: refs/heads/master';
				nextPut: Character lf ]! !

!GitRepository methodsFor: 'initialize-release' stamp: 'MaxLeske 5/14/2010 19:10'!
initializeOn: anFSReference
	reference := anFSReference! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 5/12/2010 14:13'!
isInitialized
	| isInitialized foundDirectories |
	foundDirectories := self gitDirectories collect: [:each | (self reference resolve: each) exists].
	isInitialized := foundDirectories size = (self gitDirectories size).
	isInitialized := (self reference resolve: 'HEAD') exists.
	^isInitialized! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 5/26/2010 22:25'!
isObjectDirectory: aReference
	^aReference isDirectory and: [aReference basename size = 2]! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:35'!
looseObjectBytesAt: aGitObjectSignature
	^ (self looseObjectStreamAt: aGitObjectSignature) contents! !

!GitRepository methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 10:55'!
looseObjectNames
	| r |
	r := OrderedCollection new.
	self looseObjectNamesDo: [ :n | r add: n ].
	^ r! !

!GitRepository methodsFor: 'enumerating' stamp: 'MaxLeske 5/9/2010 15:11'!
looseObjectNamesDo: aBlock 
	| root hash |
	root := self reference resolve: 'objects'.
	root childDirectories do: 
		[ :dirReference | (self isObjectDirectory: dirReference)
			ifTrue: [ dirReference childFiles do: 
				[ :fileReference | 
				hash := dirReference basename , fileReference basename.
				aBlock value: (GitObjectSignature from: hash) ] ] ]! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:33'!
looseObjectPathFor: aGitObjectSignature
	| hexHash |
	hexHash := aGitObjectSignature hexHash.
	^ 'objects/' , (hexHash first: 2) , '/' , (hexHash allButFirst: 2)! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:34'!
looseObjectStreamAt: aGitObjectSignature
	| deflatedFile |
	deflatedFile := (self reference resolve: (self looseObjectPathFor: aGitObjectSignature)) readStream.
	^ ReadStream on: (ZLibReadStream on: deflatedFile contents) contents! !

!GitRepository methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:45'!
makeTag: aGitTag
	^ self makeTagNamed: aGitTag tagName on: (self storeObject: aGitTag)! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:48'!
makeTagNamed: tagName on: aGitStorableObject
	^ self updateRef: 'refs/tags/', tagName to: aGitStorableObject signature! !

!GitRepository methodsFor: 'loading' stamp: 'MaxLeske 6/26/2010 16:48'!
objectNamed: aHexHash
	^GitObjectLoader objectNamed: aHexHash from: self! !

!GitRepository methodsFor: 'loading' stamp: 'MaxLeske 6/26/2010 16:50'!
objectWithSignature: aGitSignature
	^GitObjectLoader objectWithSignature: aGitSignature from: self! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 11:49'!
packedObjectAt: objectName
	| object |
	self packsDo: [ :each |
					object := each objectAt: objectName.
					^object ifNotNil: [^ object] ].
	^ nil! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 5/12/2010 13:48'!
packedRefs
	| f d s n |
	f := self reference resolve: 'packed-refs'.
	f exists ifFalse: [^ nil].
	
	d := Dictionary new.
	[f atEnd] whileFalse: [
		f peek = $#
			ifTrue: [f upTo: Character lf]
			ifFalse: [
				s := f upTo: Character space.
				n := f upTo: Character lf.
				d at: n put: (GitObjectSignature from: s)]].
	f close.
	^ d! !

!GitRepository methodsFor: 'enumerating' stamp: 'MaxLeske 4/26/2010 21:34'!
packsDo: aBlock
	| dir shortName |
	dir := self reference resolve: 'objects/pack'.
	dir childFiles 
		do: [ :each | (each basename endsWith: '.idx')
			ifTrue: [ shortName := each basename allButLast: 4.
					aBlock value: (GitPack new basePath: (dir fullNameFor: shortName))]]! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 4/2/2010 11:39'!
reference
	^reference! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 15:47'!
resolveLooseRef: refPath
	| target |
	target := [(self reference resolve: refPath) readStream contents asString]
		on: FSFileDoesNotExist do: [:ex | ^ nil].
	(target beginsWith: 'ref:') ifTrue: [^ self resolveRef: (target allButFirst: 4) withBlanksTrimmed].
	^ GitObjectSignature from: target substrings first "Ignore other junk after the hash."! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 15:33'!
resolveRef: name
"From the git-rev-parse manpage:
            1. if $GIT_DIR/<name> exists, that is what you mean (this is usually useful only for HEAD, FETCH_HEAD, ORIG_HEAD and MERGE_HEAD);
            2. otherwise, $GIT_DIR/refs/<name> if exists;
            3. otherwise, $GIT_DIR/refs/tags/<name> if exists;
            4. otherwise, $GIT_DIR/refs/heads/<name> if exists;
            5. otherwise, $GIT_DIR/refs/remotes/<name> if exists;
            6. otherwise, $GIT_DIR/refs/remotes/<name>/HEAD if exists."
	| fullName havePackedRefs packedRefs result |
	havePackedRefs := false.
	packedRefs := nil.
	#(	''
		'refs/'
		'refs/tags/'
		'refs/heads/'
		'refs/remotes/'
		"We don't do the last case (number 6) from the manual yet."
	) do: [:prefix |
		fullName := prefix, name.
		result := self resolveLooseRef: fullName.
		result ifNotNil: [^ result].
		havePackedRefs ifFalse: [packedRefs := self packedRefs. havePackedRefs := true].
		packedRefs ifNotNil: [
			result := packedRefs at: fullName ifAbsent: [nil].
			result ifNotNil: [^ result]]].
	^ nil! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:44'!
storeLooseObject: aGitStorableObject
	| zipStream compressed pathReference |
	zipStream := ZLibWriteStream on: (ByteArray new).
	zipStream nextPutAll: aGitStorableObject signature bytes; close.
	compressed := zipStream encodedStream contents.
	
	pathReference := self reference resolve: (self looseObjectPathFor: aGitStorableObject signature).
	pathReference parent ensureDirectory.
	pathReference writeStreamDo: [ :stream | stream nextPutAll: compressed ].
	^aGitStorableObject! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:15'!
storeObject: aGitStorableObject
	^self storeLooseObject: aGitStorableObject! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:48'!
updateRef: refPath to: aGitObjectSignature
	| target pathReference |
	target := [(self reference resolve: refPath) readStreamDo: [ :stream | stream contents ]]
		on: FSFileDoesNotExist do: [:ex | ex return: nil].

	(target isNil not and: [target beginsWith: 'ref:']) ifTrue: [
		^ self updateRef: (target allButFirst: 4) withBlanksTrimmed to: aGitObjectSignature ].

	pathReference := self reference resolve: refPath.
	pathReference parent ensureDirectory.
	pathReference writeStreamDo: [ 
		:stream | stream truncate; nextPutAll: aGitObjectSignature hexHash; nextPut: Character lf ]! !

Object subclass: #GitStamp
	instanceVariableNames: 'name email timeStamp'
	classVariableNames: 'DefaultEmail DefaultName'
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 6/25/2010 11:30'!
defaultEmail
	"^ DefaultEmail"
	^'Homer@nuke.com'! !

!GitStamp class methodsFor: 'accessing' stamp: 'lr 4/13/2010 16:20'!
defaultEmail: aString
	DefaultEmail := aString! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 6/25/2010 11:30'!
defaultName
	"^ DefaultName"
	^'Homer Simpson'! !

!GitStamp class methodsFor: 'accessing' stamp: 'lr 4/13/2010 16:21'!
defaultName: aString
	DefaultName := aString! !

!GitStamp class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 21:29'!
fromString: aString
	^ self readFrom: (ReadStream on: aString)! !

!GitStamp class methodsFor: 'instance creation' stamp: 'lr 4/13/2010 16:50'!
now
	^ self new 
		name: self defaultName; 
		email: self defaultEmail;
		yourself! !

!GitStamp class methodsFor: 'instance creation' stamp: 'MaxLeske 6/24/2010 12:29'!
readFrom: aStream
	| n e s sign z |
	n := (aStream upTo: $<) withBlanksTrimmed.
	e := (aStream upTo: $>) withBlanksTrimmed.
	aStream skipSeparators.
	s := Integer readFrom: aStream.
	aStream skipSeparators.
	sign := aStream next.
	('-+' includes: sign) ifFalse: [self error: 'Invalid timezone sign'].
	z := Integer readFrom: aStream.
	z := (z // 100) hours + (z \\ 100) minutes.
	sign = $- ifTrue: [z := z negated].
	
	^ self new
		name: n;
		email: e; 
		timeStamp: ((DateAndTime epoch + s seconds) utcOffset: z);
		yourself! !

!GitStamp methodsFor: 'private' stamp: 'tonyg 2/23/2010 21:24'!
cleanUp: aString
	^ aString copyWithoutAll: '<>', String lf! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:21'!
email
	^ email! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:23'!
email: aString
	email := self cleanUp: aString! !

!GitStamp methodsFor: 'initialize-release' stamp: 'lr 4/13/2010 16:46'!
initialize
	super initialize.
	timeStamp := TimeStamp now! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:22'!
name
	^ name! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:23'!
name: aString
	name := self cleanUp: aString! !

!GitStamp methodsFor: 'printing' stamp: 'MaxLeske 5/25/2010 20:30'!
printOn: aStream
	| z |
	aStream
		nextPutAll: name; nextPutAll: ' <'; nextPutAll: email; nextPutAll: '> ';
		nextPutAll: timeStamp asSeconds printString;
		nextPut: Character space.
	z := timeStamp offset.
	z < Duration zero
		ifTrue: [aStream nextPut: $-. z := z negated]
		ifFalse: [aStream nextPut: $+].
	aStream
		nextPutAll: (z hours printString forceTo: 2 paddingWith: $0);
		nextPutAll: (z minutes printString forceTo: 2 paddingWith: $0).! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:21'!
timeStamp
	^ timeStamp! !

!GitStamp methodsFor: 'accessing' stamp: 'MaxLeske 5/25/2010 20:30'!
timeStamp: aTimeStamp
	timeStamp := aTimeStamp! !

Object subclass: #GitStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Git-Plumbing'!

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
contents
	^stream contents! !

!GitStream methodsFor: 'initialize-release' stamp: 'MaxLeske 4/23/2010 13:45'!
initialize
	stream := WriteStream on: ByteArray new! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:51'!
lf
	self nextPut: Character lf! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:43'!
nextPut: aCharacter
	stream nextPut: aCharacter asInteger! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
nextPutAll: aString
	stream nextPutAll: aString asByteArray ! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 10:55'!
nextPutBytes: aByteArray
	stream nextPutAll: aByteArray! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:51'!
nullByte
	self nextPut: 0! !

!GitStream methodsFor: 'testing' stamp: 'MaxLeske 6/8/2010 22:33'!
size
	^stream size! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
space
	self nextPut: Character space! !
